# -*- coding: utf-8 -*-

from django.core.exceptions import ObjectDoesNotExist
from django.db import models
from django.utils.translation import ugettext_lazy as _

from apps.organization.models import Organization
from apps.address.models import Address
from thx.core.models import UserModel, DateModel, StatusModel, OrderedModel, CleanModel


class Bank(CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    """
    Store banks
    MODELS:
    :model: `organization.Organization`
    """
    organization = models.ForeignKey(
        Organization,
        on_delete=models.PROTECT,
        related_name='banks',
        verbose_name=_("organization"),
    )
    abi = models.CharField(
        max_length=5,
        db_index=True,
        verbose_name=_("ABI"),
    )
    cab = models.CharField(
        max_length=5,
        db_index=True,
        verbose_name=_("CAB"),
    )
    int_code = models.CharField(
        max_length=4,
        blank=True,
        verbose_name=_("int code")
    )
    cin = models.CharField(
        max_length=1,
        blank=True,
        verbose_name=_("CIN")
    )
    cc = models.CharField(
        max_length=12,
        blank=True,
        verbose_name=_("CC")
    )
    note = models.TextField(
        max_length=255,
        blank=True,
        verbose_name=_("note"),
    )

    class Meta:
        verbose_name = _('bank')
        verbose_name_plural = _('banks')
        ordering = ('ordering', 'abi', 'cab')
        permissions = (
            ("list_bank", "Can list bank"),
            ("detail_bank", "Can detail bank"),
            ("enable_bank", "Can enable bank"),
            ("disable_bank", "Can disable bank"),
        )

    def __str__(self):
        return self.iban

    @property
    def iban(self):
        """
        Return IBAN code
        """
        return '{int_code}{cin}{abi}{cab}{cc}'.format(**vars(self))

    def get_detail_bank(self):
        """
        Return an DetailBank instance
        """
        try:
            return BankDetail.objects.get(abi=self.abi, cab=self.cab)
        except ObjectDoesNotExist:
            return BankDetail.objects.none()


class BankDetail(Address, CleanModel, UserModel, DateModel, StatusModel, OrderedModel):
    """
    Store bank details
    """
    abi = models.CharField(
        max_length=5,
        db_index=True,
        verbose_name=_("ABI"),
    )
    cab = models.CharField(
        max_length=5,
        db_index=True,
        verbose_name=_("CAB"),
    )
    name = models.CharField(
        max_length=100,
        db_index=True,
        verbose_name=_("name"),
    )
    agency = models.CharField(
        max_length=100,
        db_index=True,
        verbose_name=_("agency"),
    )

    class Meta:
        verbose_name = _('bank detail')
        verbose_name_plural = _('bank details')
        ordering = ('ordering', 'abi', 'cab',)
        unique_together = (('abi', 'cab',),)
        permissions = (
            ("list_bankdetail", "Can list bank detail"),
            ("detail_bankdetail", "Can detail bank detail"),
            ("enable_bankdetail", "Can enable bank detail"),
            ("disable_bankdetail", "Can disable bank detail"),
        )

    def __str__(self):
        return f"{self.name} - ABI: {self.abi} CAB: {self.cab}"
